from RandomForestLearner import RandomForestLearner
import csv
import numpy as np
import pandas as pd
import math
import copy
import QSTK.qstkutil.qsdateutil as du
import datetime as dt
import QSTK.qstkutil.DataAccess as da
import QSTK.qstkutil.tsutil as tsu
import QSTK.qstkstudy.EventProfiler as ep
import matplotlib.pyplot as plt




    
def getTrainData(stocks):
    
    
    stocks=["%s"%stocks]

    print stocks
   
    start = dt.datetime(2010, 1, 1)
    end = dt.datetime(2012, 1, 1)

    timeofday = dt.timedelta(hours=16)
    timestamps = du.getNYSEdays(start, end, timeofday)
    dataobj = da.DataAccess('Yahoo')

    keys = ['open', 'high', 'low', 'close', 'volume', 'actual_close']

    data = dataobj.get_data(timestamps, stocks, keys)
    dataDic = dict(zip(keys, data))

    for key in keys:
        dataDic[key] = dataDic[key].fillna(method='ffill')
        dataDic[key] = dataDic[key].fillna(method='bfill')
        dataDic[key] = dataDic[key].fillna(1.0)
    priceTrain = dataDic['actual_close'].values

    print len(priceTrain)

    Xtrain,Ytrain,Y = train(priceTrain)
    return Xtrain,Ytrain,Y
 

 
 
def getFrequency(base):
    mean = sum(base) / len(base)
    frequency = 0
    for i in range(0, 20):
        if base[i]<=mean and base[i+1]>mean:
            frequency = frequency + 1
        elif base[i]<mean and base[i+1]>=mean:
            frequency = frequency + 1
        elif base[i]>=mean and base[i+1]<mean:
            frequency = frequency + 1
        elif base[i]>mean and base[i+1]<=mean:
            frequency = frequency + 1
    return frequency
	
def get_mean(base):
    return np.mean(base)
	
def get_relative_strength_idx(base):
    comparer = base[0]
    gain =0.0
    loss =0.0
    new_dataset= base[0:]
    for each in new_dataset:
        if each > comparer:
            gain = gain + (each-comparer)
        elif each < comparer:
            loss = loss + (comparer-each)
    if loss ==0.0:
        return 100.0
    elif gain ==0.0:
        return 0.0
    else:
        rs = (gain/len(new_dataset))/(loss/len(new_dataset))
        rsi = float(100 - (100/(1+rs)))
        return rsi




def slope(base):
    x = np.array([i for i in range(1,22)])
    y = np.array([data for data in base])
    A = np.vstack([x, np.ones(len(x))]).T
    m, c = np.linalg.lstsq(A, y)[0]
    return m

def getStdev(base):
    length = len(base)
    mean = sum(base) / length
    deviation = 0
    for i in base:
        deviation += (i - mean) ** 2
    return (deviation / length) ** 0.5
	


def getOneDayDiff(base):
    return base[20]-base[19]
	



	
	


def train(data):
    maxI = data.shape[1]
    

    Xtrain = np.zeros([maxI*(len(data)-25),11])
    print Xtrain.shape
    Ytrain = np.zeros([maxI*(len(data)-25),1])
    Y = np.zeros([maxI*(len(data)-25),1])
    print Y.shape

    count = 0
    for i in range (0,maxI):
        print "i=", i
        for j in range (0, len(data)-25):
            base = data[j:j+21, i]
            x1 = getOneDayDiff(base)
            x2 = getStdev(base)
            x3 = slope(base)
            x4 = getFrequency(base)
           
            x5=get_relative_strength_idx(base)
            x6=get_mean(base)
           
            Xtrain[count, 0] = x1
            Xtrain[count, 1] = x2
            Xtrain[count, 2] = x3 
            Xtrain[count, 3] = x4
            
            Xtrain[count, 4] = x5
            Xtrain[count, 6] = x6
            

            Ytrain[count, 0] = (data[j+25,i]-data[j+20,i])/data[j+20,i]
            Y[count, 0] = data[j+20,i]
            count = count + 1
            
    return Xtrain,Ytrain,Y
def getTestData(stocks):
    
     
    stocks=["%s"%stocks]

    print stocks
    start = dt.datetime(2012, 2, 1)
    end = dt.datetime(2012, 12, 1)

    timeofday = dt.timedelta(hours=16)
    timestamps = du.getNYSEdays(start, end, timeofday)
    dataobj = da.DataAccess('Yahoo')

    keys = ['open', 'high', 'low', 'close', 'volume', 'actual_close']
    

    data = dataobj.get_data(timestamps, stocks, keys)
    dataDic = dict(zip(keys, data))

    for key in keys:
        dataDic[key] = dataDic[key].fillna(method='ffill')
        dataDic[key] = dataDic[key].fillna(method='bfill')
        dataDic[key] = dataDic[key].fillna(1.0)
    priceTest = dataDic['actual_close'].values

    print len(priceTest)

    Xtest, Y, Ytest = train(priceTest)
   

    return Xtest, Ytest    

def RMS(ActualValue, PredictedValue):
    total = 0
    for i in range(0, len(ActualValue)):
        total = total + (ActualValue[i] - PredictedValue[i]) * (ActualValue[i]-PredictedValue[i])

        rms = math.sqrt(total / len(ActualValue))
    
    
    return rms

def Corrcoef(Actual, Predicted):
    corr = np.corrcoef(Actual, Predicted)
    return corr[0,1]

def createScatterPlot(xLabel, yLabel, xData, yData, stocks):
    plt.clf()
    fig = plt.figure()
    plt.plot(xData, yData, 'o')
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.savefig(stocks, format='pdf')

def createComparisonPlot(xLabel, yLabel, xData, y1Data, y2Data, stocks, linename):
    plt.clf()
    fig = plt.figure()
    fig.add_subplot(111)
    plt.plot(xData, y1Data, color = 'blue')
    plt.plot(xData, y2Data, color = 'red')
    plt.legend(linename)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.savefig(stocks, format='pdf')

def createComparison10Plot(xLabel, yLabel, xData, y1Data, y2Data, y3Data, y4Data, y5Data,y6Data, stocks, linename):
    plt.clf()
    fig = plt.figure()
    fig.add_subplot(111)
    plt.plot(xData, y1Data)
    plt.plot(xData, y2Data)
    plt.plot(xData, y3Data)
    plt.plot(xData, y4Data)
    plt.plot(xData, y5Data)
    plt.plot(xData, y6Data)
    
    plt.legend(linename)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.savefig(stocks, format='pdf')
    
    
def test(stocks):
    error=0.0
	
    Xtrain, Ytrain,Y = getTrainData(stocks)
    Xtest, Ytest = getTestData(stocks)
    print Xtest.shape
    Rowsize=Xtest.shape[0]            #For easier computation in the following lines.Gets the number of rows in the testing set
    X1 = Xtest[0:Rowsize, 0]
    X2 = Xtest[0:Rowsize, 1]
    X3 = Xtest[0:Rowsize, 2]
    X4 = Xtest[0:Rowsize, 3]
    X5 = Xtest[0:Rowsize, 4]
    X6 = Xtest[0:Rowsize, 5]
    
	
    ActualValue = Ytest[:,0]
    print 
    
      
    lowestRms=100
    maxK=2
    
    for k in range(2,15): #Gets extremely slow due, and higher variances may result
       
       #RandomForestLearner
       goLearn = RandomForestLearner(k)
       goLearn.ToLearn(Xtrain, Ytrain)
       PredictedValue = goLearn.query(Xtest)
       PredictedValue = PredictedValue[:,-1]
       for i in range (0, PredictedValue.shape[0]):
         PredictedValue[i] = (PredictedValue[i] + 1) * ActualValue[i]
       Rms = RMS(ActualValue, PredictedValue)
       print ("Value of k %i for which Rms is %f "%(k,Rms))
       if Rms<lowestRms:
              lowestRms=Rms
              print lowestRms
              maxK=k
              continue
              
       print "maxK %f " %maxK
                    
      
    print maxK	 
    
    
    #RandomForestLearner
    goLearn = RandomForestLearner(maxK)
    goLearn.ToLearn(Xtrain, Ytrain)
    PredictedValue = goLearn.query(Xtest)
    PredictedValue = PredictedValue[:,-1]
    writer = csv.writer(open('Predicted Value %s.csv' % stocks, 'wb'), delimiter=',')
    for i in range(0,PredictedValue.shape[0]):
        row_to_enter = [PredictedValue[i]]
        writer.writerow(row_to_enter)

    for i in range (0, PredictedValue.shape[0]):
        PredictedValue[i] = (PredictedValue[i] + 1) * ActualValue[i]
        
    

    writer = csv.writer(open('Actual and Predicted value %s.csv' % stocks, 'wb'), delimiter=',')
    for i in range(0,PredictedValue.shape[0]):
        row_to_enter = [ActualValue[i], PredictedValue[i]]
        writer.writerow(row_to_enter)

    #RMS Error(out-of-sample)
    RootMeanSquareError = RMS(ActualValue, PredictedValue)
    error=error+ RootMeanSquareError
    print "error is %f" %error

    #Correlation Coefficient
    Corr = Corrcoef(ActualValue, PredictedValue)

    print RootMeanSquareError, Corr
	
	

    writer = csv.writer(open('Root Mean Square Error and Correlation Coefficient %s.csv' % stocks, 'wb'), delimiter=',')
    row_to_enter = ['rms', RootMeanSquareError]
    writer.writerow(row_to_enter)
    row_to_enter = ['corr',Corr]
    writer.writerow(row_to_enter)

   

   

    writer = csv.writer(open('Features %s.csv' % stocks, 'wb'), delimiter=',')
    for i in range(0,Rowsize):
        row_to_enter = [X1[i],X2[i], X3[i],X4[i],X5[i],X6[i]]
        writer.writerow(row_to_enter)

    dateArray=np.zeros(ActualValue.shape[0])
    for i in range(0, ActualValue.shape[0]):
        dateArray[i] = i+1
    Date=np.zeros(ActualValue.shape[0])
    
    for i in range (0, ActualValue.shape[0]):
        Date[i] = i-1
    
    label = ['ActualValue', 'PredictedValue']
    createComparisonPlot('Days', 'Price', dateArray, ActualValue, PredictedValue, 'Comparison %s.pdf '% stocks, label)
    


    createScatterPlot('PredictedValue', 'ActualValue', PredictedValue, ActualValue, 'scatter plot %s.pdf '% stocks)
    
    label = ['One Day Diff', 'Stdev', 'slope', 'Frequency','Relative Strength Index','Mean']
    createComparison10Plot('Dates', 'Values', Date, X1, X2, X3, X4, X5,X6, 'Features %s.pdf' %stocks, label)
    return error
if __name__ == '__main__':
    stock=["AAPL","ADSK","AKAM","AMZN","CSCO","EA","INTU","NFLX","NVDA","QCOM", "MSFT","IBM"]


    
    
    for stocks in stock:
           
           rms= test(stocks)
           #error=error+rms
		   
    #print "error is %f" % (error/len(stock))
